﻿using Microsoft.OpenApi.Models;
using Swashbuckle.AspNetCore.SwaggerGen;
using System;
using System.Linq;
using System.Text.RegularExpressions;

namespace Netmash.Extensions.Swagger.Filters
{
    public class PathParamsOperationFilter : IOperationFilter
    {
        public void Apply(OpenApiOperation operation, OperationFilterContext context)
        {
            const string paramCaptureGroup = "param";

            foreach (var parameter in operation.Parameters.ToList())
            {
                if (parameter.Name.ToLowerInvariant().StartsWith("metadata"))
                {
                    operation.Parameters.Remove(parameter);
                }
            }

            var openApiPathParameters = operation.Parameters.Where(param => param.In == ParameterLocation.Path).ToList();
            var pathParamRegEx = $@"\{{(?<{paramCaptureGroup}>[^\}}]+)\}}";

            if (openApiPathParameters.Any())
            {
                var pathParameterMatches = Regex.Matches(context.ApiDescription.RelativePath, pathParamRegEx, RegexOptions.Compiled);
                var pathParameters = pathParameterMatches.Cast<Match>().Select(x => x.Groups[paramCaptureGroup].Value);

                foreach (var openApiPathParameter in openApiPathParameters)
                {
                    var correspondingPathParameter = pathParameters.FirstOrDefault(x =>
                        string.Equals(x, openApiPathParameter.Name, StringComparison.InvariantCultureIgnoreCase));

                    if (correspondingPathParameter != null)
                        openApiPathParameter.Name = correspondingPathParameter;
                }
            }
        }
    }
}
